"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getUnknownTypesDeprecations = exports.deleteUnknownTypeObjects = void 0;

var _i18n = require("@kbn/i18n");

var _check_for_unknown_docs = require("../migrations/actions/check_for_unknown_docs");

var _helpers = require("../migrations/model/helpers");

var _lib = require("../service/lib");

/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0 and the Server Side Public License, v 1; you may not use this file except
 * in compliance with, at your election, the Elastic License 2.0 or the Server
 * Side Public License, v 1.
 */
const getKnownTypes = typeRegistry => typeRegistry.getAllTypes().map(type => type.name);

const getTargetIndices = ({
  types,
  typeRegistry,
  kibanaVersion,
  kibanaIndex
}) => {
  return [...new Set(types.map(type => (0, _lib.getIndexForType)({
    type,
    typeRegistry,
    kibanaVersion,
    defaultIndex: kibanaIndex
  })))];
};

const getUnknownSavedObjects = async ({
  typeRegistry,
  esClient,
  kibanaIndex,
  kibanaVersion
}) => {
  const knownTypes = getKnownTypes(typeRegistry);
  const targetIndices = getTargetIndices({
    types: knownTypes,
    typeRegistry,
    kibanaIndex,
    kibanaVersion
  });
  const excludeRegisteredTypes = (0, _helpers.addExcludedTypesToBoolQuery)(knownTypes);
  return await (0, _check_for_unknown_docs.getAggregatedTypesDocuments)(esClient.asInternalUser, targetIndices, excludeRegisteredTypes);
};

const getUnknownTypesDeprecations = async options => {
  const deprecations = [];
  const unknownDocs = await getUnknownSavedObjects(options);

  if (unknownDocs.length) {
    deprecations.push({
      title: _i18n.i18n.translate('core.savedObjects.deprecations.unknownTypes.title', {
        defaultMessage: 'Saved objects with unknown types are present in Kibana system indices'
      }),
      message: _i18n.i18n.translate('core.savedObjects.deprecations.unknownTypes.message', {
        defaultMessage: '{objectCount, plural, one {# object} other {# objects}} with unknown types {objectCount, plural, one {was} other {were}} found in Kibana system indices. ' + 'Upgrading with unknown savedObject types is no longer supported. ' + `To ensure that upgrades will succeed in the future, either re-enable plugins or delete these documents from the Kibana indices`,
        values: {
          objectCount: unknownDocs.length
        }
      }),
      level: 'critical',
      requireRestart: false,
      deprecationType: undefined,
      // not config nor feature...
      correctiveActions: {
        manualSteps: [_i18n.i18n.translate('core.savedObjects.deprecations.unknownTypes.manualSteps.1', {
          defaultMessage: 'Enable disabled plugins then restart Kibana.'
        }), _i18n.i18n.translate('core.savedObjects.deprecations.unknownTypes.manualSteps.2', {
          defaultMessage: 'If no plugins are disabled, or if enabling them does not fix the issue, delete the documents.'
        })],
        api: {
          path: '/internal/saved_objects/deprecations/_delete_unknown_types',
          method: 'POST',
          body: {}
        }
      }
    });
  }

  return deprecations;
};

exports.getUnknownTypesDeprecations = getUnknownTypesDeprecations;

const deleteUnknownTypeObjects = async ({
  esClient,
  typeRegistry,
  kibanaIndex,
  kibanaVersion
}) => {
  const knownTypes = getKnownTypes(typeRegistry);
  const targetIndices = getTargetIndices({
    types: knownTypes,
    typeRegistry,
    kibanaIndex,
    kibanaVersion
  });
  const nonRegisteredTypesQuery = (0, _helpers.addExcludedTypesToBoolQuery)(knownTypes);
  await esClient.asInternalUser.deleteByQuery({
    index: targetIndices,
    wait_for_completion: false,
    body: {
      query: nonRegisteredTypesQuery
    }
  });
};

exports.deleteUnknownTypeObjects = deleteUnknownTypeObjects;